-- set up the database (2008)
USE tempdb
GO
IF EXISTS ( SELECT  name
            FROM    sys.databases
            WHERE   name = N'DevGroup_2008' ) 
    DROP DATABASE DevGroup_2008
GO
CREATE DATABASE DevGroup_2008
GO
USE DevGroup_2008
GO
-- ================ Identity alter needs rebuild 
CREATE TABLE dbo.tableIdentities
    ( identityField INT IDENTITY(101, 1) -- identity property, must be above 100
    , b VARCHAR(100) constraint tableIdentities_b_default default 'some data'
    )
GO
INSERT INTO dbo.tableIdentities DEFAULT VALUES 
GO
-- alter the table:
-- ??? ALTER TABLE dbo.tableIdentities ALTER COLUMN identityField INT


--------------------------------------------------------------------------------------------
-- Explore:
sp_help 'dbo.tableIdentities'

-- DEMO: Management Studio script

SELECT  *
FROM    sys.objects
WHERE   object_id = OBJECT_ID('dbo.tableIdentities')

SELECT  *
FROM    sys.columns
WHERE   object_id = OBJECT_ID('dbo.tableIdentities')

SELECT  *
FROM    sys.default_constraints
WHERE   parent_object_id = OBJECT_ID('dbo.tableIdentities')

-- 2000:
SELECT * FROM sys.sysobjects WHERE id= OBJECT_ID('dbo.tableIdentities')

-- Standard: (some limitations)
SELECT  *
FROM    INFORMATION_SCHEMA.TABLES
WHERE   TABLE_NAME = 'tableIdentities'

SELECT  *
FROM    INFORMATION_SCHEMA.TABLE_CONSTRAINTS
WHERE   TABLE_NAME = 'tableIdentities'

SELECT  *
FROM    INFORMATION_SCHEMA.COLUMNS
WHERE   TABLE_NAME = 'tableIdentities'


----------------------------------------------------------------
----------------------------------------------------------------
-- Textual objects

CREATE PROC dbo.myProcedure1 (@paramter1 int)
AS 
    BEGIN
		-- I will execute myProcedure2, but this does not 
        -- yet exist. It will be created later, promise.
        EXECUTE dbo.myProcedure2 
    END
GO

-- Explore
-- 2005:
SELECT  *
FROM    sys.objects
WHERE   object_id = OBJECT_ID('dbo.myProcedure1')

SELECT  *
FROM    sys.parameters
WHERE   object_id = OBJECT_ID('dbo.myProcedure1')

-- 2000:
SELECT  *
FROM    sys.sysobjects
WHERE   id = OBJECT_ID('dbo.myProcedure1')

SELECT  *
FROM    sys.syscolumns
WHERE   id = OBJECT_ID('dbo.myProcedure1')

-- Standard: (some limitations)
SELECT  *
FROM    INFORMATION_SCHEMA.ROUTINES
WHERE   ROUTINE_NAME = 'myProcedure1'

SELECT  *
FROM    INFORMATION_SCHEMA.PARAMETERS
WHERE   SPECIFIC_NAME = 'myProcedure1'

-- the contents:
-- 2005
SELECT  *
FROM    sys.sql_modules
WHERE   object_id = OBJECT_ID('dbo.myProcedure1')

-- 2000
SELECT  *
FROM    sys.syscomments
WHERE   id = OBJECT_ID('dbo.myProcedure1') 

-- sysdepends:
CREATE PROC dbo.myProcedure2 (@paramter1 int)
AS 
    BEGIN
		-- I will execute myProcedure1, I know this makes no sense, 
		-- but other will know this only at runtime
        EXECUTE dbo.myProcedure1
    END
GO
-- show Management Studio
-- 2000
SELECT  *
FROM    sysdepends 

-- 2005 -- Note that the dependency on myProcerure2 is missing
SELECT  so1.[name]
      , so2.name
FROM    sys.sql_dependencies AS sd
        JOIN sys.objects AS so1 ON sd.object_id = so1.object_id
        JOIN sys.objects AS so2 ON sd.referenced_major_id = so2.object_id

ALTER PROC dbo.myProcedure2 (@paramter1 int)
AS 
    BEGIN
		-- I now have no dependency on myProcedure1
        select * from dbo.tableIdentities
    END
GO

SELECT  so1.[name]
      , so2.name
FROM    sys.sql_dependencies AS sd
        JOIN sys.objects AS so1 ON sd.object_id = so1.object_id
        JOIN sys.objects AS so2 ON sd.referenced_major_id = so2.object_id


---------------------------------------------------------------------------
-- sp_refresh
--drop table viewColumnsTable
--drop view viewColumns
CREATE TABLE dbo.viewColumnsTable
    ( a CHAR NOT NULL
    , b CHAR NOT NULL
    )
GO
INSERT  INTO dbo.viewColumnsTable
VALUES  ( 'a', 'b' )
GO
SELECT  *
FROM    dbo.viewColumnsTable
GO
CREATE VIEW dbo.viewColumns
AS  SELECT  *
    FROM    dbo.viewColumnsTable
GO
SELECT  *
FROM    dbo.viewColumns
GO
DROP TABLE viewColumnsTable
GO
CREATE TABLE dbo.viewColumnsTable
    ( b CHAR NOT NULL
    , a CHAR NOT NULL
    )
GO
INSERT  INTO dbo.viewColumnsTable
VALUES  ( 'b', 'a' )
GO
SELECT  *
FROM    dbo.viewColumnsTable
SELECT  *
FROM    dbo.viewColumns
 -- column names are different!!!
GO
sp_refreshview 'dbo.viewColumns'
GO
SELECT  *
FROM    dbo.viewColumns

--------------------------------------------------------------
--------------------------------------------------------------
-- sp_rename is bad!!!
SELECT  definition
FROM    sys.sql_modules
WHERE   object_id = OBJECT_ID('dbo.myProcedure2')
GO
sp_rename 'myProcedure2', 'myProcedure2Renamed'
GO
SELECT  definition
FROM    sys.sql_modules
WHERE   object_id = OBJECT_ID('dbo.myProcedure2Renamed')
GO
